from __future__ import annotations

import numpy as np

def det_cython(a: np.ndarray) -> complex: ...
def solve_cython(a: np.ndarray, b: np.ndarray) -> tuple[complex, complex, complex]: ...
def det4_cython(a: np.ndarray) -> complex: ...
def deriv_det4_cython(a: np.ndarry, b: np.ndarray) -> complex: ...
def deriv_det2_cython(a: np.ndarry, b: np.ndarray) -> complex: ...
